@@ -1,26 +1,21 @@
 /*
-  Copyright (C) 2002 Amir Szekely <kichik@netvision.net.il>
-
-  This software is provided 'as-is', without any express or implied
-  warranty.  In no event will the authors be held liable for any damages
-  arising from the use of this software.
-
-  Permission is granted to anyone to use this software for any purpose,
-  including commercial applications, and to alter it and redistribute it
-  freely, subject to the following restrictions:
-
-  1. The origin of this software must not be misrepresented; you must not
-  claim that you wrote the original software. If you use this software
-  in a product, an acknowledgment in the product documentation would be
-  appreciated but is not required.
-
-  2. Altered source versions must be plainly marked as such, and must not be
-  misrepresented as being the original software.
-
-  3. This notice may not be removed or altered from any source distribution.
-*/
+ * DialogTemplate.cpp
+ * 
+ * This file is a part of NSIS.
+ * 
+ * Copyright (C) 2002-2016 Amir Szekely <kichik@netvision.net.il> and Contributors
+ * 
+ * Licensed under the zlib/libpng license (the "License");
+ * you may not use this file except in compliance with the License.
+ * 
+ * Licence details can be found in the file COPYING.
+ * 
+ * This software is provided 'as-is', without any express or implied
+ * warranty.
+ */
 
 #include "DialogTemplate.h"
+#include "winchar.h"
 #include <cassert> // for assert(3)
 #ifndef _WIN32
 #  include "util.h" // for Unicode conversion functions
@@ -36,38 +31,36 @@ using namespace std;
 // Utilities
 //////////////////////////////////////////////////////////////////////
 
+static inline DWORD ConvertEndianness(DWORD d) {
+  return FIX_ENDIAN_INT32(d);
+}
+
+static inline WORD ConvertEndianness(WORD w) {
+  return FIX_ENDIAN_INT16(w);
+}
+
+static inline short ConvertEndianness(short s) {
+  return ConvertEndianness(WORD(s));
+}
+
 #define ALIGN(dwToAlign, dwAlignOn) dwToAlign = (dwToAlign%dwAlignOn == 0) ? dwToAlign : dwToAlign - (dwToAlign%dwAlignOn) + dwAlignOn
 
-// Reads a variany length array from seeker into readInto and advances seeker
-void ReadVarLenArr(LPBYTE &seeker, char* &readInto, unsigned int uCodePage) {
+// Reads a variant length array from seeker into readInto and advances seeker
+void ReadVarLenArr(LPBYTE &seeker, WINWCHAR* &readInto, unsigned int uCodePage) {
   WORD* arr = (WORD*)seeker;
-  switch (arr[0]) {
+  switch (ConvertEndianness(arr[0])) {
   case 0x0000:
     readInto = 0;
     seeker += sizeof(WORD);
     break;
   case 0xFFFF:
-    readInto = MAKEINTRESOURCE(arr[1]);
+    readInto = MAKEINTRESOURCEWINW(ConvertEndianness(arr[1]));
     seeker += 2*sizeof(WORD);
     break;
   default:
     {
-      int iStrLen = WideCharToMultiByte(uCodePage, 0, (WCHAR*)arr, -1, 0, 0, 0, 0);
-      if (iStrLen)
-      {
-        readInto = new char[iStrLen];
-        if (!WideCharToMultiByte(uCodePage, 0, (WCHAR*)arr, -1, readInto, iStrLen, 0, 0))
-        {
-          delete [] readInto;
-          throw runtime_error("ReadVarLenArr - Unicode conversion failed.");
-        }
-      }
-      else
-      {
-        throw runtime_error("ReadVarLenArr - Unicode conversion failed.");
-      }
-
-      PWCHAR wseeker = PWCHAR(seeker);
+      readInto = WinWStrDupFromWinWStr((WINWCHAR*) arr);
+      WINWCHAR *wseeker = (WINWCHAR*) seeker;
       while (*wseeker++);
       seeker = LPBYTE(wseeker);
     }
@@ -81,28 +74,26 @@ void ReadVarLenArr(LPBYTE &seeker, char* &readInto, unsigned int uCodePage) {
     if (IS_INTRESOURCE(x)) { \
       *(WORD*)seeker = 0xFFFF; \
       seeker += sizeof(WORD); \
-      *(WORD*)seeker = WORD(DWORD(x)); \
+      *(WORD*)seeker = ConvertEndianness(WORD((ULONG_PTR)(x))); \
       seeker += sizeof(WORD); \
     } \
     else { \
-      int us = MultiByteToWideChar(m_uCodePage, 0, x, -1, (WCHAR*)seeker, dwSize - DWORD(seeker - pbDlg)); \
-      if (!us) { \
-        throw runtime_error("WriteStringOrId - Unicode conversion failed."); \
-      } \
-      seeker += us*sizeof(WCHAR); \
+      WinWStrCpy((WINWCHAR *) seeker, x); \
+      seeker += WinWStrLen((WINWCHAR *) seeker) * sizeof(WINWCHAR) + sizeof(WINWCHAR); \
     } \
   else \
     seeker += sizeof(WORD);
 
 // A macro that adds the size of x (which can be a string a number, or nothing) to dwSize
-#define AddStringOrIdSize(x) dwSize += x ? (IS_INTRESOURCE(x) ? sizeof(DWORD) : MultiByteToWideChar(m_uCodePage, 0, x, -1, 0, 0) * sizeof(WCHAR)) : sizeof(WORD)
+#define AddStringOrIdSize(x) dwSize += x ? (IS_INTRESOURCE(x) ? sizeof(DWORD) : (DWORD)((WinWStrLen(x) + 1) * sizeof(WINWCHAR))) : sizeof(WORD)
 
 //////////////////////////////////////////////////////////////////////
 // Construction/Destruction
 //////////////////////////////////////////////////////////////////////
 
-CDialogTemplate::CDialogTemplate(BYTE* pbData, unsigned int uCodePage) {
+CDialogTemplate::CDialogTemplate(BYTE* pbData, bool build_unicode, unsigned int uCodePage) {
   m_uCodePage = uCodePage;
+  m_build_unicode = build_unicode;
 
   m_dwHelpId = 0;
   m_szClass = 0;
@@ -121,29 +112,29 @@ CDialogTemplate::CDialogTemplate(BYTE* pbData, unsigned int uCodePage) {
 
     DLGTEMPLATEEX* dTemplateEx = (DLGTEMPLATEEX*)pbData;
 
-    m_dwHelpId = dTemplateEx->helpID;
-    m_dwStyle = dTemplateEx->style;
-    m_dwExtStyle = dTemplateEx->exStyle;
-    m_sX = dTemplateEx->x;
-    m_sY = dTemplateEx->y;
-    m_sWidth = dTemplateEx->cx;
-    m_sHeight = dTemplateEx->cy;
+    m_dwHelpId = ConvertEndianness(dTemplateEx->helpID);
+    m_dwStyle = ConvertEndianness(dTemplateEx->style);
+    m_dwExtStyle = ConvertEndianness(dTemplateEx->exStyle);
+    m_sX = ConvertEndianness(dTemplateEx->x);
+    m_sY = ConvertEndianness(dTemplateEx->y);
+    m_sWidth = ConvertEndianness(dTemplateEx->cx);
+    m_sHeight = ConvertEndianness(dTemplateEx->cy);
 
-    wItems = dTemplateEx->cDlgItems;
+    wItems = ConvertEndianness(dTemplateEx->cDlgItems);
   }
   else {
     m_bExtended = false;
 
     DLGTEMPLATE* dTemplate = (DLGTEMPLATE*)pbData;
 
-    m_dwStyle = dTemplate->style;
-    m_dwExtStyle = dTemplate->dwExtendedStyle;
-    m_sX = dTemplate->x;
-    m_sY = dTemplate->y;
-    m_sWidth = dTemplate->cx;
-    m_sHeight = dTemplate->cy;
+    m_dwStyle = ConvertEndianness(dTemplate->style);
+    m_dwExtStyle = ConvertEndianness(dTemplate->dwExtendedStyle);
+    m_sX = ConvertEndianness(dTemplate->x);
+    m_sY = ConvertEndianness(dTemplate->y);
+    m_sWidth = ConvertEndianness(dTemplate->cx);
+    m_sHeight = ConvertEndianness(dTemplate->cy);
 
-    wItems = dTemplate->cdit;
+    wItems = ConvertEndianness(dTemplate->cdit);
   }
 
   BYTE* seeker = pbData + (m_bExtended ? sizeof(DLGTEMPLATEEX) : sizeof(DLGTEMPLATE));
@@ -156,10 +147,10 @@ CDialogTemplate::CDialogTemplate(BYTE* pbData, unsigned int uCodePage) {
   ReadVarLenArr(seeker, m_szTitle, m_uCodePage);
   // Read font size and variant length array (only if style DS_SETFONT is used!)
   if (m_dwStyle & DS_SETFONT) {
-    m_sFontSize = *(short*)seeker;
+    m_sFontSize = ConvertEndianness(*(short*)seeker);
     seeker += sizeof(short);
     if (m_bExtended) {
-      m_sFontWeight = *(short*)seeker;
+      m_sFontWeight = ConvertEndianness(*(short*)seeker);
       seeker += sizeof(short);
       m_bItalic = *(BYTE*)seeker;
       seeker += sizeof(BYTE);
@@ -171,7 +162,7 @@ CDialogTemplate::CDialogTemplate(BYTE* pbData, unsigned int uCodePage) {
 
   // Read items
   for (int i = 0; i < wItems; i++) {
-    // DLGITEMTEMPLATE[EX]s must be aligned on DWORD boundry
+    // DLGITEMTEMPLATE[EX]s must be aligned on DWORD boundary
     if (DWORD(seeker - pbData) % sizeof(DWORD))
       seeker += sizeof(WORD);
 
@@ -181,14 +172,14 @@ CDialogTemplate::CDialogTemplate(BYTE* pbData, unsigned int uCodePage) {
     if (m_bExtended) {
       DLGITEMTEMPLATEEX* rawItem = (DLGITEMTEMPLATEEX*)seeker;
 
-      item->dwHelpId = rawItem->helpID;
-      item->dwStyle = rawItem->style;
-      item->dwExtStyle = rawItem->exStyle;
-      item->sX = rawItem->x;
-      item->sY = rawItem->y;
-      item->sWidth = rawItem->cx;
-      item->sHeight = rawItem->cy;
-      item->wId = rawItem->id;
+      item->dwHelpId = ConvertEndianness(rawItem->helpID);
+      item->dwStyle = ConvertEndianness(rawItem->style);
+      item->dwExtStyle = ConvertEndianness(rawItem->exStyle);
+      item->sX = ConvertEndianness(rawItem->x);
+      item->sY = ConvertEndianness(rawItem->y);
+      item->sWidth = ConvertEndianness(rawItem->cx);
+      item->sHeight = ConvertEndianness(rawItem->cy);
+      item->wId = ConvertEndianness(rawItem->id);
 
       seeker += sizeof(DLGITEMTEMPLATEEX);
     }
@@ -196,13 +187,13 @@ CDialogTemplate::CDialogTemplate(BYTE* pbData, unsigned int uCodePage) {
       DLGITEMTEMPLATE* rawItem = (DLGITEMTEMPLATE*)seeker;
 
       item->dwHelpId = 0;
-      item->dwStyle = rawItem->style;
-      item->dwExtStyle = rawItem->dwExtendedStyle;
-      item->sX = rawItem->x;
-      item->sY = rawItem->y;
-      item->sWidth = rawItem->cx;
-      item->sHeight = rawItem->cy;
-      item->wId = rawItem->id;
+      item->dwStyle = ConvertEndianness(rawItem->style);
+      item->dwExtStyle = ConvertEndianness(rawItem->dwExtendedStyle);
+      item->sX = ConvertEndianness(rawItem->x);
+      item->sY = ConvertEndianness(rawItem->y);
+      item->sWidth = ConvertEndianness(rawItem->cx);
+      item->sHeight = ConvertEndianness(rawItem->cy);
+      item->wId = ConvertEndianness(rawItem->id);
 
       seeker += sizeof(DLGITEMTEMPLATE);
     }
@@ -214,7 +205,7 @@ CDialogTemplate::CDialogTemplate(BYTE* pbData, unsigned int uCodePage) {
 
     // Read creation data variant length array
     // First read the size of the array (no null termination)
-    item->wCreateDataSize = *(WORD*)seeker;
+    item->wCreateDataSize = ConvertEndianness(*(WORD*)seeker);
     seeker += sizeof(WORD);
     // Then read the array it self (if size is not 0)
     if (item->wCreateDataSize) {
@@ -229,23 +220,22 @@ CDialogTemplate::CDialogTemplate(BYTE* pbData, unsigned int uCodePage) {
   }
 }
 
+static void free_template_string(WINWCHAR*s)
+{
+  if (!IS_INTRESOURCE(s)) free(s);
+}
+
 CDialogTemplate::~CDialogTemplate() {
-  if (m_szMenu && !IS_INTRESOURCE(m_szMenu))
-    delete [] m_szMenu;
-  if (m_szClass && !IS_INTRESOURCE(m_szClass))
-    delete [] m_szClass;
-  if (m_szTitle)
-    delete [] m_szTitle;
-  if (m_szFont)
-    delete [] m_szFont;
+  free_template_string(m_szMenu);
+  free_template_string(m_szClass);
+  free(m_szTitle);
+  free(m_szFont);
 
   for (unsigned int i = 0; i < m_vItems.size(); i++) {
-    if (m_vItems[i]->szClass && !IS_INTRESOURCE(m_vItems[i]->szClass))
-      delete [] m_vItems[i]->szClass;
-    if (m_vItems[i]->szTitle && !IS_INTRESOURCE(m_vItems[i]->szTitle))
-      delete [] m_vItems[i]->szTitle;
-    if (m_vItems[i]->szCreationData)
-      delete [] m_vItems[i]->szCreationData;
+    free_template_string(m_vItems[i]->szClass);
+    free_template_string(m_vItems[i]->szTitle);
+    if (m_vItems[i]->szCreationData) delete [] m_vItems[i]->szCreationData;
+    delete m_vItems[i];
   }
 }
 
@@ -290,20 +280,14 @@ int CDialogTemplate::RemoveItem(WORD wId) {
 }
 
 // Sets the font of the dialog
-void CDialogTemplate::SetFont(char* szFaceName, WORD wFontSize) {
-  if (strcmp(szFaceName, "MS Shell Dlg")) {
-     // not MS Shell Dlg
-    m_dwStyle &= ~DS_SHELLFONT;
-  }
-  else {
-    // MS Shell Dlg
+void CDialogTemplate::SetFont(TCHAR* szFaceName, WORD wFontSize) {
+  m_dwStyle &= ~DS_SHELLFONT;
+  if (!_tcscmp(szFaceName, _T("MS Shell Dlg"))) // TODO: "MS Shell Dlg 2"?
     m_dwStyle |= DS_SHELLFONT;
-  }
-  m_bCharset = DEFAULT_CHARSET;
   m_dwStyle |= DS_SETFONT;
-  if (m_szFont) delete [] m_szFont;
-  m_szFont = new char[strlen(szFaceName)+1];
-  strcpy(m_szFont, szFaceName);
+  m_bCharset = DEFAULT_CHARSET;
+  free(m_szFont);
+  m_szFont = WinWStrDupFromTChar(szFaceName, m_uCodePage);
   m_sFontSize = wFontSize;
 }
 
@@ -313,12 +297,10 @@ void CDialogTemplate::AddItem(DialogItemTemplate item) {
   CopyMemory(newItem, &item, sizeof(DialogItemTemplate));
 
   if (item.szClass && !IS_INTRESOURCE(item.szClass)) {
-    newItem->szClass = new char[strlen(item.szClass)+1];
-    strcpy(newItem->szClass, item.szClass);
+    newItem->szClass = WinWStrDupFromWinWStr(item.szClass);
   }
   if (item.szTitle && !IS_INTRESOURCE(item.szTitle)) {
-    newItem->szTitle = new char[strlen(item.szTitle)+1];
-    strcpy(newItem->szTitle, item.szTitle);
+    newItem->szTitle = WinWStrDupFromWinWStr(item.szTitle);
   }
   if (item.wCreateDataSize) {
     newItem->szCreationData = new char[item.wCreateDataSize];
@@ -347,7 +329,7 @@ HWND CDialogTemplate::CreateDummyDialog() {
   DWORD dwTemp;
   BYTE* pbDlg = Save(dwTemp);
   HWND hDlg = CreateDialogIndirect(GetModuleHandle(0), (DLGTEMPLATE*)pbDlg, 0, 0);
-  delete [] pbDlg;
+  FreeSavedTemplate(pbDlg);
   if (!hDlg)
     throw runtime_error("Can't create dialog from template!");
 
@@ -365,6 +347,12 @@ void CDialogTemplate::PixelsToDlgUnits(short& x, short& y) {
   y = short(float(y) / (float(r.bottom)/10000));
 }
 
+void CDialogTemplate::PixelsToDlgUnits(SIZE& siz) {
+  short x = (short)siz.cx, y = (short)siz.cy;
+  PixelsToDlgUnits(x, y);
+  siz.cx = x, siz.cy = y;
+}
+
 // Converts pixels to this dialog's units
 void CDialogTemplate::DlgUnitsToPixels(short& x, short& y) {
   HWND hDlg = CreateDummyDialog();
@@ -377,7 +365,7 @@ void CDialogTemplate::DlgUnitsToPixels(short& x, short& y) {
 }
 
 // Returns the size of a string in the dialog (in dialog units)
-SIZE CDialogTemplate::GetStringSize(WORD id, char *str) {
+SIZE CDialogTemplate::GetStringSize(WORD id, TCHAR *str) {
   HWND hDlg = CreateDummyDialog();
 
   LOGFONT f;
@@ -388,19 +376,19 @@ SIZE CDialogTemplate::GetStringSize(WORD id, char *str) {
   SelectObject(memDC, font);
 
   SIZE size;
-  GetTextExtentPoint32(memDC, str, strlen(str), &size);
+  GetTextExtentPoint32(memDC, str, (int) _tcslen(str), &size);
 
   DestroyWindow(hDlg);
   DeleteObject(font);
   DeleteDC(memDC);
 
-  PixelsToDlgUnits((short&)size.cx, (short&)size.cy);
-  
+  PixelsToDlgUnits(size);
+
   return size;
 }
 
 // Trims the right margins of a control to fit a given text string size.
-void CDialogTemplate::RTrimToString(WORD id, char *str, int margins) {
+void CDialogTemplate::RTrimToString(WORD id, TCHAR *str, int margins) {
   DialogItemTemplate* item = GetItem(id);
   if (!item) return;
 
@@ -414,7 +402,7 @@ void CDialogTemplate::RTrimToString(WORD id, char *str, int margins) {
 }
 
 // Trims the left margins of a control to fit a given text string size.
-void CDialogTemplate::LTrimToString(WORD id, char *str, int margins) {
+void CDialogTemplate::LTrimToString(WORD id, TCHAR *str, int margins) {
   DialogItemTemplate* item = GetItem(id);
   if (!item) return;
 
@@ -429,7 +417,7 @@ void CDialogTemplate::LTrimToString(WORD id, char *str, int margins) {
 }
 
 // Trims the left and right margins of a control to fit a given text string size.
-void CDialogTemplate::CTrimToString(WORD id, char *str, int margins) {
+void CDialogTemplate::CTrimToString(WORD id, TCHAR *str, int margins) {
   DialogItemTemplate* item = GetItem(id);
   if (!item) return;
 
@@ -448,9 +436,10 @@ void CDialogTemplate::CTrimToString(WORD id, char *str, int margins) {
 void CDialogTemplate::ConvertToRTL() {
   for (unsigned int i = 0; i < m_vItems.size(); i++) {
     bool addExStyle = false;
+    bool addExLeftScrollbar = true;
 
     // Button
-    if (long(m_vItems[i]->szClass) == 0x80) {
+    if ((ULONG_PTR)(m_vItems[i]->szClass) == 0x80) {
       m_vItems[i]->dwStyle ^= BS_LEFTTEXT;
       m_vItems[i]->dwStyle ^= BS_RIGHT;
       m_vItems[i]->dwStyle ^= BS_LEFT;
@@ -464,13 +453,13 @@ void CDialogTemplate::ConvertToRTL() {
       }
     }
     // Edit
-    else if (long(m_vItems[i]->szClass) == 0x81) {
+    else if ((ULONG_PTR)(m_vItems[i]->szClass) == 0x81) {
       if ((m_vItems[i]->dwStyle & ES_CENTER) == 0) {
         m_vItems[i]->dwStyle ^= ES_RIGHT;
       }
     }
     // Static
-    else if (long(m_vItems[i]->szClass) == 0x82) {
+    else if ((ULONG_PTR)(m_vItems[i]->szClass) == 0x82) {
       if ((m_vItems[i]->dwStyle & SS_TYPEMASK) == SS_LEFT || (m_vItems[i]->dwStyle & SS_TYPEMASK) == SS_LEFTNOWORDWRAP)
       {
         m_vItems[i]->dwStyle &= ~SS_TYPEMASK;
@@ -480,21 +469,29 @@ void CDialogTemplate::ConvertToRTL() {
         m_vItems[i]->dwStyle |= SS_CENTERIMAGE;
       }
     }
-    else if (!IS_INTRESOURCE(m_vItems[i]->szClass) && !stricmp(m_vItems[i]->szClass, "RichEdit20A")) {
+    else if (!IS_INTRESOURCE(m_vItems[i]->szClass) && !WinWStrNICmpASCII(m_vItems[i]->szClass, "RichEdit20", 10)) {
       if ((m_vItems[i]->dwStyle & ES_CENTER) == 0) {
         m_vItems[i]->dwStyle ^= ES_RIGHT;
       }
     }
-    else if (!IS_INTRESOURCE(m_vItems[i]->szClass) && !stricmp(m_vItems[i]->szClass, "SysTreeView32")) {
+    else if (!IS_INTRESOURCE(m_vItems[i]->szClass) && !WinWStrICmpASCII(m_vItems[i]->szClass, "SysTreeView32")) {
       m_vItems[i]->dwStyle |= TVS_RTLREADING;
+      m_vItems[i]->dwExtStyle |= WS_EX_LAYOUTRTL;
       addExStyle = true;
+      addExLeftScrollbar = false;
+    }
+    else if (!IS_INTRESOURCE(m_vItems[i]->szClass) && !WinWStrICmpASCII(m_vItems[i]->szClass, "SysListView32")) {
+      m_vItems[i]->dwExtStyle |= WS_EX_LAYOUTRTL;
+      addExLeftScrollbar = false;
     }
     else addExStyle = true;
 
     if (addExStyle)
       m_vItems[i]->dwExtStyle |= WS_EX_RIGHT;
+    if (addExLeftScrollbar)
+      m_vItems[i]->dwExtStyle |= WS_EX_LEFTSCROLLBAR;
 
-    m_vItems[i]->dwExtStyle |= WS_EX_RTLREADING | WS_EX_LEFTSCROLLBAR;
+    m_vItems[i]->dwExtStyle |= WS_EX_RTLREADING;
 
     m_vItems[i]->sX = m_sWidth - m_vItems[i]->sWidth - m_vItems[i]->sX;
   }
@@ -511,16 +508,16 @@ BYTE* CDialogTemplate::Save(DWORD& dwSize) {
 
   if (m_bExtended) {
     DLGTEMPLATEEX dh = {
-      0x0001,
-      0xFFFF,
-      m_dwHelpId,
-      m_dwExtStyle,
-      m_dwStyle,
-      m_vItems.size(),
-      m_sX,
-      m_sY,
-      m_sWidth,
-      m_sHeight
+      ConvertEndianness(WORD(0x0001)),
+      ConvertEndianness(WORD(0xFFFF)),
+      ConvertEndianness(m_dwHelpId),
+      ConvertEndianness(m_dwExtStyle),
+      ConvertEndianness(m_dwStyle),
+      ConvertEndianness(WORD(m_vItems.size())),
+      ConvertEndianness(m_sX),
+      ConvertEndianness(m_sY),
+      ConvertEndianness(m_sWidth),
+      ConvertEndianness(m_sHeight)
     };
 
     CopyMemory(seeker, &dh, sizeof(DLGTEMPLATEEX));
@@ -528,13 +525,13 @@ BYTE* CDialogTemplate::Save(DWORD& dwSize) {
   }
   else {
     DLGTEMPLATE dh = {
-      m_dwStyle,
-      m_dwExtStyle,
-      m_vItems.size(),
-      m_sX,
-      m_sY,
-      m_sWidth,
-      m_sHeight
+      ConvertEndianness(m_dwStyle),
+      ConvertEndianness(m_dwExtStyle),
+      ConvertEndianness(WORD(m_vItems.size())),
+      ConvertEndianness(m_sX),
+      ConvertEndianness(m_sY),
+      ConvertEndianness(m_sWidth),
+      ConvertEndianness(m_sHeight)
     };
 
     CopyMemory(seeker, &dh, sizeof(DLGTEMPLATE));
@@ -550,10 +547,10 @@ BYTE* CDialogTemplate::Save(DWORD& dwSize) {
 
   // Write font variant length array, size, and extended info (if needed)
   if (m_dwStyle & DS_SETFONT) {
-    *(short*)seeker = m_sFontSize;
+    *(short*)seeker = ConvertEndianness(m_sFontSize);
     seeker += sizeof(short);
     if (m_bExtended) {
-      *(short*)seeker = m_sFontWeight;
+      *(short*)seeker = ConvertEndianness(m_sFontWeight);
       seeker += sizeof(short);
       *(BYTE*)seeker = m_bItalic;
       seeker += sizeof(BYTE);
@@ -565,20 +562,20 @@ BYTE* CDialogTemplate::Save(DWORD& dwSize) {
 
   // Write all of the items
   for (unsigned int i = 0; i < m_vItems.size(); i++) {
-    // DLGITEMTEMPLATE[EX]s must be aligned on DWORD boundry
-    if (DWORD(seeker - pbDlg) % sizeof(DWORD))
+    // DLGITEMTEMPLATE[EX]s must be aligned on DWORD boundary
+    if ((seeker - pbDlg) % sizeof(DWORD))
       seeker += sizeof(WORD);
 
     if (m_bExtended) {
       DLGITEMTEMPLATEEX dih = {
-        m_vItems[i]->dwHelpId,
-        m_vItems[i]->dwExtStyle,
-        m_vItems[i]->dwStyle,
-        m_vItems[i]->sX,
-        m_vItems[i]->sY,
-        m_vItems[i]->sWidth,
-        m_vItems[i]->sHeight,
-        m_vItems[i]->wId
+        ConvertEndianness(m_vItems[i]->dwHelpId),
+        ConvertEndianness(m_vItems[i]->dwExtStyle),
+        ConvertEndianness(m_vItems[i]->dwStyle),
+        ConvertEndianness(m_vItems[i]->sX),
+        ConvertEndianness(m_vItems[i]->sY),
+        ConvertEndianness(m_vItems[i]->sWidth),
+        ConvertEndianness(m_vItems[i]->sHeight),
+        ConvertEndianness(m_vItems[i]->wId)
       };
 
       CopyMemory(seeker, &dih, sizeof(DLGITEMTEMPLATEEX));
@@ -586,13 +583,13 @@ BYTE* CDialogTemplate::Save(DWORD& dwSize) {
     }
     else {
       DLGITEMTEMPLATE dih = {
-        m_vItems[i]->dwStyle,
-        m_vItems[i]->dwExtStyle,
-        m_vItems[i]->sX,
-        m_vItems[i]->sY,
-        m_vItems[i]->sWidth,
-        m_vItems[i]->sHeight,
-        m_vItems[i]->wId
+        ConvertEndianness(m_vItems[i]->dwStyle),
+        ConvertEndianness(m_vItems[i]->dwExtStyle),
+        ConvertEndianness(m_vItems[i]->sX),
+        ConvertEndianness(m_vItems[i]->sY),
+        ConvertEndianness(m_vItems[i]->sWidth),
+        ConvertEndianness(m_vItems[i]->sHeight),
+        ConvertEndianness(m_vItems[i]->wId)
       };
 
       CopyMemory(seeker, &dih, sizeof(DLGITEMTEMPLATE));
@@ -600,7 +597,17 @@ BYTE* CDialogTemplate::Save(DWORD& dwSize) {
     }
 
     // Write class variant length array
-    WriteStringOrId(m_vItems[i]->szClass);
+    const WINWCHAR *szClass = m_vItems[i]->szClass;
+#ifdef _UNICODE
+    static const WINWCHAR clsRE20W[] = {'R','i','c','h','E','d','i','t','2','0','W',0};
+    static const WINWCHAR clsRE20A[] = {'R','i','c','h','E','d','i','t','2','0','A',0};
+    if (!IS_INTRESOURCE(szClass)) {
+      // transmute RichEdit20A/W control into RichEdit20T that matches the target
+      if (m_build_unicode && !WinWStrICmpASCII(szClass, "RichEdit20A")) szClass = clsRE20W;
+      if (!m_build_unicode && !WinWStrICmpASCII(szClass, "RichEdit20W")) szClass = clsRE20A;
+    }
+#endif
+    WriteStringOrId(szClass);
     // Write title variant length array
     WriteStringOrId(m_vItems[i]->szTitle);
 
@@ -608,7 +615,7 @@ BYTE* CDialogTemplate::Save(DWORD& dwSize) {
     // First write its size
     WORD wCreateDataSize = m_vItems[i]->wCreateDataSize;
     if (m_vItems[i]->wCreateDataSize) wCreateDataSize += sizeof(WORD);
-    *(WORD*)seeker = wCreateDataSize;
+    *(WORD*)seeker = ConvertEndianness(wCreateDataSize);
     seeker += sizeof(WORD);
     // If size is nonzero write the data too
     if (m_vItems[i]->wCreateDataSize) {
@@ -617,7 +624,7 @@ BYTE* CDialogTemplate::Save(DWORD& dwSize) {
     }
   }
 
-  assert((DWORD) seeker - (DWORD) pbDlg == dwSize);
+  assert((DWORD)(seeker - pbDlg) == dwSize);
 
   // DONE!
   return pbDlg;
@@ -641,7 +648,7 @@ DWORD CDialogTemplate::GetSize() {
   }
 
   for (unsigned int i = 0; i < m_vItems.size(); i++) {
-    // DLGITEMTEMPLATE[EX]s must be aligned on DWORD boundry
+    // DLGITEMTEMPLATE[EX]s must be aligned on DWORD boundary
     ALIGN(dwSize, sizeof(DWORD));
 
     dwSize += m_bExtended ? sizeof(DLGITEMTEMPLATEEX) : sizeof(DLGITEMTEMPLATE);
