@@ -1,72 +1,198 @@
+/*
+ * Platform.h
+ * 
+ * This file is a part of NSIS.
+ * 
+ * Copyright (C) 1999-2016 Nullsoft and Contributors
+ * 
+ * Licensed under the zlib/libpng license (the "License");
+ * you may not use this file except in compliance with the License.
+ * 
+ * Licence details can be found in the file COPYING.
+ * 
+ * This software is provided 'as-is', without any express or implied
+ * warranty.
+ *
+ * Unicode support by Jim Park -- 08/13/2007
+ */
+
 #ifndef ___PLATFORM__H___
 #define ___PLATFORM__H___
 
 // some definitions for non Win32 platforms were taken from MinGW's free Win32 library
 
+
+#if defined(__cplusplus) && defined(MAKENSIS) && (!defined(_MSC_VER) || _MSC_VER > 1200)
+template<class T> class NSISCHARTYPE{ T _c; public: NSISCHARTYPE(){} NSISCHARTYPE(T c):_c(c){} operator T()const{ return _c; } };
+typedef NSISCHARTYPE<unsigned short> WINWCHAR; // WINWCHAR is always UTF16LE and should not be passed to wcs* functions
+#else
+typedef unsigned short WINWCHAR;
+#endif
+
+
 // includes
 
+#include "tchar.h"
+
 #ifdef _WIN32
-#  include <windows.h>
-#  include <commctrl.h>
+#include <windows.h>
+#include <commctrl.h>
+#include <shellapi.h>
+#include <shlwapi.h>
+#include <shlobj.h>
 #else
+#  include <stdint.h>
 #  ifndef EXEHEAD
 #    include <string.h>
 #    include <stdlib.h>
 #  endif
 // basic types
-typedef unsigned char BYTE, *PBYTE, *LPBYTE;
-typedef unsigned short WORD, *LPWORD;
-typedef unsigned long DWORD, *LPDWORD;
-typedef short SHORT;
-typedef unsigned short USHORT;
-typedef unsigned int UINT;
-typedef unsigned int UINT32;
-typedef int INT;
-typedef int INT32;
-typedef long LONG;
-typedef unsigned long ULONG;
-typedef long long INT64, LARGE_INTEGER;
-typedef unsigned long long UINT64, ULARGE_INTEGER;
+typedef uint8_t BYTE, *PBYTE, *LPBYTE;
+typedef uint16_t WORD, *LPWORD;
+typedef uint32_t DWORD, *LPDWORD;
+typedef int16_t SHORT;
+typedef uint16_t USHORT;
+typedef uint32_t UINT;
+typedef uint32_t UINT32;
+typedef int32_t INT;
+typedef int32_t INT32;
+typedef int32_t LONG;
+typedef uint32_t ULONG;
+typedef int64_t INT64, LARGE_INTEGER;
+typedef uint64_t UINT64, ULARGE_INTEGER;
+typedef intptr_t INT_PTR;
+typedef uintptr_t UINT_PTR;
 typedef int BOOL, *LPBOOL;
+typedef short VARIANT_BOOL;
 typedef void VOID;
 typedef void *LPVOID;
 typedef char CHAR, *PCHAR, *LPCH, *PCH, *NPSTR, *LPSTR, *PSTR;
 typedef unsigned char UCHAR;
 typedef const char *LPCCH, *PCSTR, *LPCSTR;
-typedef unsigned short WCHAR, *PWCHAR, *LPWCH, *PWCH, *NWPSTR, *LPWSTR, *PWSTR;
-typedef const unsigned short *LPCWCH, *PCWCH, *LPCWSTR, *PCWSTR;
-typedef unsigned int UINT_PTR;
+typedef unsigned short WCHAR, OLECHAR, *PWCHAR, *LPWCH, *PWCH, *NWPSTR, *LPWSTR, *PWSTR, *BSTR;
+typedef const unsigned short *LPCWCH, *PCWCH, *LPCWSTR, *PCWSTR, *LPCOLESTR;
+#ifndef _tctime
+#define _tctime _wctime
+#else
+#define _tctime ctime
+#endif
 // basic stuff
-typedef unsigned long HANDLE;
+typedef void * HANDLE;
+typedef HANDLE HWND;
+typedef HANDLE HMODULE;
 typedef unsigned long HKEY;
 // some gdi
-typedef unsigned long COLORREF;
-typedef unsigned long HBRUSH;
+typedef DWORD COLORREF;
+typedef HANDLE HBRUSH;
 // bool
 #  define FALSE 0
 #  define TRUE 1
 // more
 typedef WORD LANGID;
+// ULONGLONG
+#ifdef __GNUC__
+#define _HAVE_INT64
+#define _INTEGRAL_MAX_BITS 64
+#undef __int64
+#define __int64 long long
+#elif defined(__WATCOMC__) && (defined(_INTEGRAL_MAX_BITS) && _INTEGRAL_MAX_BITS >= 64 )
+#define _HAVE_INT64
+#endif /* __GNUC__/__WATCOMC */
+#if defined(_HAVE_INT64) || (defined(_INTEGRAL_MAX_BITS) && _INTEGRAL_MAX_BITS >= 64)
+typedef __int64 LONGLONG;
+typedef unsigned __int64 DWORDLONG;
+#else
+typedef double LONGLONG,DWORDLONG;
+#endif
+typedef LONGLONG *PLONGLONG;
+typedef DWORDLONG *PDWORDLONG;
+typedef DWORDLONG ULONGLONG,*PULONGLONG;
+
+// function mapping
+#define _strdup strdup
+#define _snprintf snprintf
+#define _vsnprintf vsnprintf
+
+#endif // ?WIN32
+
+#ifndef INT_MAX
+#include <limits.h>
 #endif
 
 
+// mingw32 and w64-mingw32 do not define ULONG_PTR
+// but rather declare ULONG_PTR via typedef (see basetsd.h)
+#if !defined(__MINGW32__) && !defined(ULONG_PTR)
+#  ifndef _WIN64
+#    define ULONG_PTR unsigned long
+#  endif
+#endif
+
+#ifdef __cplusplus
+#include <algorithm>
+#if defined(_MSC_VER) && ( _MSC_VER <= 1200 || (defined(_MIN) && _MSC_FULL_VER >= 140000000 && _MSC_FULL_VER <= 140040310) )
+#define STD_MIN std::_MIN
+#define STD_MAX std::_MAX
+#else
+#define STD_MIN (std::min) // This works even when windows.h defines min/max
+#define STD_MAX (std::max)
+#endif
+#endif
+
+#ifndef COUNTOF
+#define COUNTOF(a) (sizeof(a)/sizeof(a[0]))
+#endif
+
+#ifndef __BIG_ENDIAN__
+# define FIX_ENDIAN_INT64(x) (x)
+# define FIX_ENDIAN_INT32_INPLACE(x) ((void)(x))
+# define FIX_ENDIAN_INT32(x) (x)
+# define FIX_ENDIAN_INT16_INPLACE(x) ((void)(x))
+# define FIX_ENDIAN_INT16(x) (x)
+#else
+# define FIX_ENDIAN_INT64(x) SWAP_ENDIAN_INT64(x)
+# define FIX_ENDIAN_INT32_INPLACE(x) ((x) = SWAP_ENDIAN_INT32(x))
+# define FIX_ENDIAN_INT32(x) SWAP_ENDIAN_INT32(x)
+# define FIX_ENDIAN_INT16_INPLACE(x) ((x) = SWAP_ENDIAN_INT16(x))
+# define FIX_ENDIAN_INT16(x) SWAP_ENDIAN_INT16(x)
+#endif
+#define SWAP_ENDIAN_INT64(x) ( \
+  (((x)&0xFF00000000000000) >> 56) | \
+  (((x)&0x00FF000000000000) >> 40) | \
+  (((x)&0x0000FF0000000000) >> 24) | \
+  (((x)&0x000000FF00000000) >>  8) | \
+  (((x)&0x00000000FF000000) <<  8) | \
+  (((x)&0x0000000000FF0000) << 24) | \
+  (((x)&0x000000000000FF00) << 40) | \
+  (((x)&0x00000000000000FF) << 56) )
+#define SWAP_ENDIAN_INT32(x) ( \
+  (((x)&0xFF000000) >> 24) | \
+  (((x)&0x00FF0000) >>  8) | \
+  (((x)&0x0000FF00) <<  8) | \
+  (((x)&0x000000FF) << 24) )
+#define SWAP_ENDIAN_INT16(x) ( \
+  (((x)&0xFF00) >> 8) | \
+  (((x)&0x00FF) << 8) )
+
 // script path separator
 
-#  define PATH_SEPARATOR_STR "\\"
-#  define PATH_SEPARATOR_C '\\'
+#  define PATH_SEPARATOR_STR _T("\\")
+#  define PATH_SEPARATOR_C _T('\\')
 
 // system specific characters
 
 #ifdef _WIN32
-#  define PLATFORM_PATH_SEPARATOR_STR "\\"
-#  define PLATFORM_PATH_SEPARATOR_C '\\'
-#  define OPT_STR "/"
-#  define OPT_C '/'
+#  define PLATFORM_PATH_SEPARATOR_STR _T("\\")
+#  define PLATFORM_PATH_SEPARATOR_C _T('\\')
+#  define OPT_STR _T("/")
+#  define OPT_C _T('/')
+#  define IS_OPT(a) (a[0]==OPT_C||a[0]==_T('-'))
 #else
-#  define PLATFORM_PATH_SEPARATOR_STR "/"
-#  define PLATFORM_PATH_SEPARATOR_C '/'
-#  define OPT_STR "-"
-#  define OPT_C '-'
+#  define PLATFORM_PATH_SEPARATOR_STR _T("/")
+#  define PLATFORM_PATH_SEPARATOR_C _T('/')
+#  define OPT_STR _T("-")
+#  define OPT_C _T('-')
+#  define IS_OPT(a) (a[0]==OPT_C)
 #endif
 
 // attributes
@@ -85,52 +211,66 @@ typedef WORD LANGID;
 #  endif
 #endif
 
-// Added by Dave Laundon 19th August 2002
-// For all internal functions, use of stdcall calling convention moves the
-// responsibility for tidying the stack to callee from caller, reducing the code
-// involved considerably.  Gives an instant saving of 0.5K.
-// NB - the zlib and bzip2 portions have been given the same treatment, but with
-// project compiler-options settings and/or project-wide defines.
-// NB - safer for NSIS's routines to be defined explicitly to avoid problems
-// calling DLL functions.
-#if defined(_WIN32) && ((_MSC_VER >= 800) || defined(_STDCALL_SUPPORTED))
-#  define NSISCALL  __stdcall   // Ordinary functions
-#  define NSISCALLV __cdecl     // Variable-argument-list functions
+#if defined(__GNUC__)
+#define UNUSED __attribute__((unused))
 #else
-#  if defined(__GNUC__) && defined(__i386__)
-#    define NSISCALL  __attribute__((__stdcall__))   // Ordinary functions
-#    define NSISCALLV __attribute__((__cdecl__))     // Variable-argument-list functions
-#  else
-#    define NSISCALL
-#    define NSISCALLV
-#  endif
+#define UNUSED
 #endif
 
 // macros
 
 #ifndef _WIN32
 #  ifndef FIELD_OFFSET
-#    define FIELD_OFFSET(t,f) ((LONG)&(((t*)0)->f))
+#    define FIELD_OFFSET(t,f) ((UINT_PTR)&(((t*)0)->f))
+#  endif
+#  ifndef MAKEINTRESOURCEA
+#    define MAKEINTRESOURCEA(i) ((LPSTR)((ULONG_PTR)((WORD)(i))))
+#  endif
+#  ifndef MAKEINTRESOURCEW
+#    define MAKEINTRESOURCEW(i) ((LPWSTR)((ULONG_PTR)((WORD)(i))))
 #  endif
 #  ifndef MAKEINTRESOURCE
-#    define MAKEINTRESOURCE(i) (LPSTR)((DWORD)((WORD)(i)))
+#    define MAKEINTRESOURCE MAKEINTRESOURCEA
 #  endif
 #  ifndef IMAGE_FIRST_SECTION
-#    define IMAGE_FIRST_SECTION(h) ((PIMAGE_SECTION_HEADER) ((DWORD)h+FIELD_OFFSET(IMAGE_NT_HEADERS,OptionalHeader)+((PIMAGE_NT_HEADERS)(h))->FileHeader.SizeOfOptionalHeader))
+#    define IMAGE_FIRST_SECTION(h) ( PIMAGE_SECTION_HEADER( (ULONG_PTR) h + \
+                                     FIELD_OFFSET(IMAGE_NT_HEADERS, OptionalHeader) + \
+                                     FIX_ENDIAN_INT16(PIMAGE_NT_HEADERS(h)->FileHeader.SizeOfOptionalHeader) ) )
 #  endif
 #  ifndef RGB
 #    define RGB(r,g,b) ((DWORD)(((BYTE)(r)|((WORD)(g)<<8))|(((DWORD)(BYTE)(b))<<16)))
 #  endif
+#  ifndef LOBYTE
+#    define LOBYTE(w) ((BYTE)(w))
+#    define HIBYTE(w) ((BYTE)(((WORD)(w)>>8)&0xFF))
+#  endif
+#  ifndef MAKEWORD
+#    define MAKEWORD(a,b) ((WORD)(((BYTE)(a))|(((WORD)((BYTE)(b)))<<8)))
+#  endif
 #  ifndef MAKELONG
-#    define MAKELONG(a,b) ((LONG)(((WORD)(a))|(((DWORD)((WORD)(b)))<<16)))
+#    define MAKELONG(a,b) ((DWORD)(((WORD)(a))|(((DWORD)((WORD)(b)))<<16)))
 #  endif
 #endif
 #ifndef IS_INTRESOURCE
 #  define IS_INTRESOURCE(_r) (((ULONG_PTR)(_r) >> 16) == 0)
 #endif
 
+#ifndef IS_HIGH_SURROGATE
+#  define IS_HIGH_SURROGATE(wch) (((wch) >= 0xd800) && ((wch) <= 0xdbff))
+#endif
+
 // functions
 
+// Anders: MSVC's swprintf is non standard, use _snwprintf when you really mean swprintf
+#if !defined(_MSC_VER) && !defined(__MINGW32__) && !defined(_snwprintf)
+#  define _snwprintf swprintf // (wchar_t*,size_t,const wchar_t*,...)
+#endif
+#ifndef _WIN32
+#  define _vsnwprintf vswprintf // (wchar_t*,size_t,const wchar_t*,va_list)
+#endif
+
+// Jim Park: These str functions will probably never be encountered with all my
+// Unicode changes.  And if they were used, these would probably be wrong.
 #ifndef _WIN32
 #  define stricmp strcasecmp
 #  define strcmpi strcasecmp
@@ -139,14 +279,21 @@ typedef WORD LANGID;
 #  define ZeroMemory(x, y) memset(x, 0, y)
 #endif
 
-// defines
-
-#ifndef FOF_NOERRORUI
-#  define FOF_NOERRORUI 0x0400
+#ifndef _WIN64
+#  ifndef GCLP_HICON
+#    define GCLP_HICON GCL_HICON
+#    define SetClassLongPtr SetClassLong
+#  endif
 #endif
 
-#ifndef ULONG_PTR
-#  define ULONG_PTR DWORD
+// defines
+
+#ifndef MEM_LARGE_PAGES
+#  define MEM_LARGE_PAGES 0x20000000
+#endif
+
+#ifndef WC_NO_BEST_FIT_CHARS
+#  define WC_NO_BEST_FIT_CHARS 0x400
 #endif
 
 #ifndef IDC_HAND
@@ -175,10 +322,13 @@ typedef WORD LANGID;
 
 #ifndef EXEHEAD
 #  ifndef SF_TEXT
-#    define SF_TEXT 1
+#    define SF_TEXT 0x0001
 #  endif
 #  ifndef SF_RTF
-#    define SF_RTF 2
+#    define SF_RTF 0x0002
+#  endif
+#  ifndef SF_UNICODE
+#    define SF_UNICODE 0x0010
 #  endif
 #endif
 
@@ -186,7 +336,7 @@ typedef WORD LANGID;
 #  undef INVALID_FILE_ATTRIBUTES
 #endif
 #ifndef INVALID_FILE_ATTRIBUTES
-#  define INVALID_FILE_ATTRIBUTES ((unsigned long) -1)
+#  define INVALID_FILE_ATTRIBUTES ((DWORD) -1)
 #endif
 
 // shell folders
@@ -332,6 +482,10 @@ typedef WORD LANGID;
 #  define CSIDL_CDBURN_AREA 0x3B
 #endif
 
+#ifndef SHGFP_TYPE_CURRENT
+  #define SHGFP_TYPE_CURRENT 0
+#endif
+
 // other shell stuff
 
 #ifndef SHACF_FILESYSTEM
@@ -342,6 +496,10 @@ typedef WORD LANGID;
 
 #ifndef CP_ACP
 #  define CP_ACP 0
+#  define CP_OEMCP 1
+#endif
+#ifndef CP_UTF8
+#  define CP_UTF8 65001
 #endif
 
 #ifndef COLOR_BTNFACE
@@ -350,6 +508,9 @@ typedef WORD LANGID;
 #ifndef COLOR_WINDOW
 #  define COLOR_WINDOW 5
 #endif
+#ifndef COLOR_HOTLIGHT
+#  define COLOR_HOTLIGHT 26
+#endif
 
 // resources
 
@@ -398,6 +559,7 @@ typedef WORD LANGID;
 #  define MB_ICONQUESTION 32
 #  define MB_ICONEXCLAMATION 48
 #  define MB_ICONINFORMATION 64
+#  define MB_USERICON 128
 #  define MB_SETFOREGROUND 0x10000
 #  define MB_TOPMOST 0x40000
 #  define MB_RIGHT 0x80000
@@ -449,6 +611,7 @@ typedef WORD LANGID;
 #  define WS_EX_RIGHTSCROLLBAR 0
 #  define WS_EX_RTLREADING 0x2000
 #  define WS_EX_LEFTSCROLLBAR 0x4000
+#  define WS_EX_LAYOUTRTL 0x00400000
 
 #  define TVS_RTLREADING 64
 
@@ -479,6 +642,17 @@ typedef WORD LANGID;
 #  define HKEY_DYN_DATA ((HKEY)0x80000006)
 #endif
 
+#ifndef KEY_WOW64_64KEY
+#  define KEY_WOW64_64KEY 0x100
+#endif
+
+#ifndef REG_SZ
+#  define REG_SZ 1
+#  define REG_EXPAND_SZ 2
+#  define REG_BINARY 3
+#  define REG_DWORD 4
+#endif
+
 // show modes
 
 #ifndef SW_SHOWNORMAL
@@ -487,8 +661,11 @@ typedef WORD LANGID;
 #  define SW_SHOWMINIMIZED 2
 #  define SW_SHOWMAXIMIZED 3
 #  define SW_SHOWNOACTIVATE 4
+#  define SW_SHOW 5
 #  define SW_SHOWMINNOACTIVE 7
 #  define SW_SHOWNA 8
+#  define SW_RESTORE 9
+#  define SW_SHOWDEFAULT 10
 #endif
 
 // hotkeys
@@ -547,6 +724,9 @@ typedef WORD LANGID;
 #  define FOF_SIMPLEPROGRESS 256
 #  define FOF_NOCONFIRMMKDIR 512
 #endif
+#ifndef FOF_NOERRORUI
+#  define FOF_NOERRORUI 0x0400
+#endif
 
 // file attribs
 
@@ -560,14 +740,6 @@ typedef WORD LANGID;
 #  define FILE_ATTRIBUTE_OFFLINE 0x00001000
 #endif
 
-// registry
-
-#ifndef REG_SZ
-#  define REG_SZ 1
-#  define REG_EXPAND_SZ 2
-#  define REG_BINARY 3
-#  define REG_DWORD 4
-#endif
 
 // fopen
 #ifndef GENERIC_READ
@@ -594,12 +766,29 @@ typedef WORD LANGID;
 
 #ifndef _WIN32
 #  define IMAGE_NUMBEROF_DIRECTORY_ENTRIES 16
-#  define IMAGE_DOS_SIGNATURE 0x5A4D
-#  define IMAGE_NT_SIGNATURE 0x00004550
+#  ifndef __BIG_ENDIAN__
+#    define IMAGE_DOS_SIGNATURE 0x5A4D
+#    define IMAGE_NT_SIGNATURE 0x00004550
+#  else
+#    define IMAGE_DOS_SIGNATURE 0x4D5A
+#    define IMAGE_NT_SIGNATURE 0x50450000
+#  endif
 #  define IMAGE_FILE_DLL 8192
 #  define IMAGE_DIRECTORY_ENTRY_EXPORT 0
 #  define IMAGE_SIZEOF_SHORT_NAME 8
 #endif
+#ifndef IMAGE_DLLCHARACTERISTICS_DYNAMIC_BASE
+#define IMAGE_DLLCHARACTERISTICS_DYNAMIC_BASE 0x0040
+#endif
+#ifndef IMAGE_DLLCHARACTERISTICS_NX_COMPAT
+#define IMAGE_DLLCHARACTERISTICS_NX_COMPAT 0x0100
+#endif
+#ifndef IMAGE_DLLCHARACTERISTICS_NO_SEH
+#define IMAGE_DLLCHARACTERISTICS_NO_SEH 0x0400
+#endif
+#ifndef IMAGE_DLLCHARACTERISTICS_TERMINAL_SERVER_AWARE
+#define IMAGE_DLLCHARACTERISTICS_TERMINAL_SERVER_AWARE 0x8000
+#endif
 
 // structures
 
@@ -618,7 +807,7 @@ typedef struct _LOGFONT {
   BYTE lfClipPrecision;
   BYTE lfQuality;
   BYTE lfPitchAndFamily;
-  CHAR lfFaceName[LF_FACESIZE];
+  TCHAR lfFaceName[LF_FACESIZE];
 } LOGFONT;
 #  pragma pack(2)
 typedef struct _IMAGE_DOS_HEADER {
@@ -642,6 +831,7 @@ typedef struct _IMAGE_DOS_HEADER {
   WORD e_res2[10];
   LONG e_lfanew;
 } IMAGE_DOS_HEADER,*PIMAGE_DOS_HEADER;
+#  pragma pack()
 #  pragma pack(4)
 typedef struct _IMAGE_FILE_HEADER {
   WORD Machine;
@@ -656,7 +846,7 @@ typedef struct _IMAGE_DATA_DIRECTORY {
   DWORD VirtualAddress;
   DWORD Size;
 } IMAGE_DATA_DIRECTORY,*PIMAGE_DATA_DIRECTORY;
-typedef struct _IMAGE_OPTIONAL_HEADER {
+typedef struct _IMAGE_OPTIONAL_HEADER32 {
   WORD Magic;
   BYTE MajorLinkerVersion;
   BYTE MinorLinkerVersion;
@@ -688,12 +878,67 @@ typedef struct _IMAGE_OPTIONAL_HEADER {
   DWORD LoaderFlags;
   DWORD NumberOfRvaAndSizes;
   IMAGE_DATA_DIRECTORY DataDirectory[IMAGE_NUMBEROF_DIRECTORY_ENTRIES];
-} IMAGE_OPTIONAL_HEADER,*PIMAGE_OPTIONAL_HEADER;
-typedef struct _IMAGE_NT_HEADERS {
+} IMAGE_OPTIONAL_HEADER32,*PIMAGE_OPTIONAL_HEADER32;
+typedef struct _IMAGE_OPTIONAL_HEADER64 {
+  WORD Magic;
+  BYTE MajorLinkerVersion;
+  BYTE MinorLinkerVersion;
+  DWORD SizeOfCode;
+  DWORD SizeOfInitializedData;
+  DWORD SizeOfUninitializedData;
+  DWORD AddressOfEntryPoint;
+  DWORD BaseOfCode;
+  ULONGLONG ImageBase;
+  DWORD SectionAlignment;
+  DWORD FileAlignment;
+  WORD MajorOperatingSystemVersion;
+  WORD MinorOperatingSystemVersion;
+  WORD MajorImageVersion;
+  WORD MinorImageVersion;
+  WORD MajorSubsystemVersion;
+  WORD MinorSubsystemVersion;
+  DWORD Win32VersionValue;
+  DWORD SizeOfImage;
+  DWORD SizeOfHeaders;
+  DWORD CheckSum;
+  WORD Subsystem;
+  WORD DllCharacteristics;
+  ULONGLONG SizeOfStackReserve;
+  ULONGLONG SizeOfStackCommit;
+  ULONGLONG SizeOfHeapReserve;
+  ULONGLONG SizeOfHeapCommit;
+  DWORD LoaderFlags;
+  DWORD NumberOfRvaAndSizes;
+  IMAGE_DATA_DIRECTORY DataDirectory[IMAGE_NUMBEROF_DIRECTORY_ENTRIES];
+} IMAGE_OPTIONAL_HEADER64,*PIMAGE_OPTIONAL_HEADER64;
+typedef struct _IMAGE_NT_HEADERS32 {
   DWORD Signature;
   IMAGE_FILE_HEADER FileHeader;
-  IMAGE_OPTIONAL_HEADER OptionalHeader;
-} IMAGE_NT_HEADERS,*PIMAGE_NT_HEADERS;
+  IMAGE_OPTIONAL_HEADER32 OptionalHeader;
+} IMAGE_NT_HEADERS32,*PIMAGE_NT_HEADERS32;
+typedef struct _IMAGE_NT_HEADERS64 {
+  DWORD Signature;
+  IMAGE_FILE_HEADER FileHeader;
+  IMAGE_OPTIONAL_HEADER64 OptionalHeader;
+} IMAGE_NT_HEADERS64,*PIMAGE_NT_HEADERS64;
+#ifdef _WIN64
+typedef IMAGE_OPTIONAL_HEADER64  IMAGE_OPTIONAL_HEADER;
+typedef PIMAGE_OPTIONAL_HEADER64 PIMAGE_OPTIONAL_HEADER;
+typedef IMAGE_NT_HEADERS64 IMAGE_NT_HEADERS;
+typedef PIMAGE_NT_HEADERS64 PIMAGE_NT_HEADERS;
+#else
+typedef IMAGE_OPTIONAL_HEADER32  IMAGE_OPTIONAL_HEADER;
+typedef PIMAGE_OPTIONAL_HEADER32 PIMAGE_OPTIONAL_HEADER;
+typedef IMAGE_NT_HEADERS32  IMAGE_NT_HEADERS;
+typedef PIMAGE_NT_HEADERS32 PIMAGE_NT_HEADERS;
+#endif
+#ifndef __BIG_ENDIAN__
+#  define IMAGE_NT_OPTIONAL_HDR32_MAGIC 0x010b
+#  define IMAGE_NT_OPTIONAL_HDR64_MAGIC 0x020b
+#else
+#  define IMAGE_NT_OPTIONAL_HDR32_MAGIC 0x0b01
+#  define IMAGE_NT_OPTIONAL_HDR64_MAGIC 0x0b02
+#endif
 typedef struct _IMAGE_SECTION_HEADER {
   BYTE Name[IMAGE_SIZEOF_SHORT_NAME];
  union {
@@ -722,7 +967,104 @@ typedef struct _IMAGE_EXPORT_DIRECTORY {
   DWORD AddressOfNames;
   DWORD AddressOfNameOrdinals;
 } IMAGE_EXPORT_DIRECTORY,*PIMAGE_EXPORT_DIRECTORY;
+typedef struct tagVS_FIXEDFILEINFO {
+  DWORD dwSignature;
+  DWORD dwStrucVersion;
+  DWORD dwFileVersionMS;
+  DWORD dwFileVersionLS;
+  DWORD dwProductVersionMS;
+  DWORD dwProductVersionLS;
+  DWORD dwFileFlagsMask;
+  DWORD dwFileFlags;
+  DWORD dwFileOS;
+  DWORD dwFileType;
+  DWORD dwFileSubtype;
+  DWORD dwFileDateMS;
+  DWORD dwFileDateLS;
+} VS_FIXEDFILEINFO;
 #  pragma pack()
 #endif
 
+
+// MinGW does not implement the unicode CRT startup functions
+#if (defined(_UNICODE) && defined(_WIN32)) && defined(__MINGW32__)
+#  define NSIS_ENTRYPOINT_TMAIN \
+    int _tmain(int argc,WCHAR**argv); \
+    EXTERN_C int main(int ac,char**cav) { \
+      WCHAR**av=CommandLineToArgvW(GetCommandLineW(),&ac); \
+      if (!av) { \
+        _tprintf(_T("wmain: Error %u\n"),ac = GetLastError()); \
+        return ac; \
+      } \
+      ac = _tmain(ac,av); \
+      /*LEAK: LocalFree(av);*/ \
+      return ac; \
+    }
+#  define NSIS_ENTRYPOINT_SIMPLEGUI \
+     int WINAPI _tWinMain(HINSTANCE hI,HINSTANCE hOld,LPTSTR cl,int sc); \
+     EXTERN_C int WINAPI WinMain(HINSTANCE hI,HINSTANCE hOld,char*cl,int sc) \
+     {return _tWinMain(hI,0,0,sc);}
+#  ifdef __cplusplus
+#    define NSIS_ENTRYPOINT_GUINOCRT \
+       EXTERN_C void NSISWinMainNOCRT(); \
+       int WINAPI WinMain(HINSTANCE hI,HINSTANCE hOld,char*cl,int sc) \
+       {NSISWinMainNOCRT();return 0;}
+#  endif
 #endif
+#ifndef NSIS_ENTRYPOINT_TMAIN
+#  define NSIS_ENTRYPOINT_TMAIN
+#endif
+#ifndef NSIS_ENTRYPOINT_SIMPLEGUI // _tWinMain with valid hInstance, calls ExitProcess
+#  define NSIS_ENTRYPOINT_SIMPLEGUI
+#endif
+#ifndef NSIS_ENTRYPOINT_GUINOCRT
+#  define NSIS_ENTRYPOINT_GUINOCRT
+#endif
+
+
+#define NSIS_CXX_THROWSPEC(ignoredthrowspec) // Ignore c++ exception specifications
+#define BUGBUG64TRUNCATE(cast,xpr) ( (cast) (xpr) )
+
+/*
+_tprintf on Windows/MSVCRT treats %s as TCHAR* and on POSIX %s is always char*!
+Always use our NPRI* (NsisPRInt*[Narrow|Wide]) defines in format strings when calling 
+functions from tchar.h (Similar to the way <inttypes.h> works)
+
+Example: _tprintf(_T("Hello %") NPRIs _T("\n"), _T("World"));
+*/
+#ifdef _WIN32
+#  define NPRIs _T("s")
+#  define NPRIns _T("hs")
+#  define NPRIws _T("ls") // ws also works, not sure which is most compatible
+#  ifndef _WIN64
+#    define NPRIp _T(".8x")
+#    define NPRIpN ".8x"
+#  endif
+#else  // !_WIN32
+#  define NPRIns _T("s")
+#  define NPRIws _T("ls")
+#  ifdef _UNICODE
+#    define NPRIs _T("ls")
+#  else // !_UNICODE
+#    define NPRIs _T("s")
+#  endif // ~_UNICODE
+#endif // ~_WIN32
+#ifndef NPRIp
+#  define NPRIp _T("p")
+#  define NPRIpN "p"
+#endif
+
+
+// Disable deprecated warnings (Windows SDK for Windows 8.1)
+#ifdef _MSC_VER
+#if _MSC_VER >= 1500
+FORCEINLINE DWORD NoDepr_GetVersion() { __pragma(warning(push))__pragma(warning(disable:4996)) DWORD r = GetVersion(); __pragma(warning(pop)) return r; }
+#define GetVersion NoDepr_GetVersion
+FORCEINLINE BOOL NoDepr_GetVersionExA(OSVERSIONINFOA*p) { __pragma(warning(push))__pragma(warning(disable:4996)) BOOL r = GetVersionExA(p); __pragma(warning(pop)) return r; }
+#define GetVersionExA NoDepr_GetVersionExA
+FORCEINLINE BOOL NoDepr_GetVersionExW(OSVERSIONINFOW*p) { __pragma(warning(push))__pragma(warning(disable:4996)) BOOL r = GetVersionExW(p); __pragma(warning(pop)) return r; }
+#define GetVersionExW NoDepr_GetVersionExW
+#endif //~ _MSC_VER >= 1500
+#endif //~ _MSC_VER
+
+#endif // EOF
